/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "field.h"

#include "../etc/draw.h"
#include "../etc/dirs.h"

/*********************************************************************
 *                            Constructor                            *
 *********************************************************************/
field::field()
{
}

/*********************************************************************
 *                             Destructor                            *
 *********************************************************************/
field::~field()
{
}

/*********************************************************************
 *                            createField                            *
 *********************************************************************/
void field::createField(string fileName)
{
   dirs dir;

   /* Load Texture */
   if(!publicities)
   {
      glGenTextures(1,&texture);
   }
   SDL_Surface* tex = IMG_Load(dir.getRealFile(fileName).c_str());
   setTexture(tex, texture, true);

   GLfloat maxAni=1.0;
   glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &maxAni);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, maxAni);

   texturePropX = (float)tex->w / (float)smallestPowerOfTwo(tex->w);
   texturePropY = (float)tex->h / (float)smallestPowerOfTwo(tex->h);
   SDL_FreeSurface(tex);

   /* Set values FIXME -> better load from definitions file! */
   sizeX = FIELD_X;
   sizeZ = FIELD_Z;
   sideDelta = FIELD_SIDE_DELTA;
   littleAreaDeltaX = FIELD_LITTLE_AREA_DELTA_X;
   littleAreaDeltaZ = FIELD_LITTLE_AREA_DELTA_Z;
   penaltyMarkDelta = FIELD_PENALTY_MARK_DELTA;

   /* Put Some Publicities */
   if(publicities)
   {
      delete(publicities);
   }
   publicities = new fieldPub();
   publicities->addPublicity("models/publicity/model.cfg", 
                             "models/publicity/dnt/texture.png",
                             sizeX-63, 0, -6, 0); 
   publicities->addPublicity("models/publicity/model.cfg", 
                             "models/publicity/pingus/texture.png",
                             63, 0, -6, 0); 
   publicities->addPublicity("models/publicity/model.cfg", 
                             "models/publicity/dccnitghtmare/texture.png",
                             124, 0, -6, 0); 
   /*publicities->addPublicity("models/publicity/model.cfg", 
                             "models/publicity/scummvm/texture.png",
                             185, 0, -6, 0); 
   publicities->addPublicity("models/publicity/model.cfg", 
                             "models/publicity/scavenger/texture.png",
                             246, 0, -6, 0); 
   publicities->addPublicity("models/publicity/model.cfg", 
                             "models/publicity/scourge/texture.png",
                             307, 0, -6, 0);
   publicities->addPublicity("models/publicity/model.cfg", 
                             "models/publicity/supertuxkart/texture.png",
                             368, 0, -6, 0);
   publicities->addPublicity("models/publicity/model.cfg", 
                             "models/publicity/supertuxkart/texture.png",
                             429, 0, -6, 0); 
   publicities->addPublicity("models/publicity/model.cfg", 
                             "models/publicity/supertuxkart/texture.png",
                             490, 0, -6, 0); 
   publicities->addPublicity("models/publicity/model.cfg", 
                             "models/publicity/supertuxkart/texture.png",
                             551, 0, -6, 0); 
   publicities->addPublicity("models/publicity/model.cfg", 
                             "models/publicity/supertuxkart/texture.png",
                             612, 0, -6, 0);*/  
}

/*********************************************************************
 *                            deleteField                            *
 *********************************************************************/
void field::deleteField()
{
   glDeleteTextures(1,&texture);
   delete(publicities);
}

/*********************************************************************
 *                               render                              *
 *********************************************************************/
void field::render()
{
   /* Render a big plane bellow ground to avoid errors
    * on mouse projection out of the field */
   glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
   glBegin(GL_QUADS);
      glVertex3f(-FARVIEW, -5.0f, -FARVIEW);
      glVertex3f(-FARVIEW, -5.0f, sizeZ+FARVIEW);
      glVertex3f(sizeX+FARVIEW, -5.0f, sizeZ+FARVIEW);
      glVertex3f(sizeX+FARVIEW, -5.0f, -FARVIEW);
   glEnd();
   glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);

   /* Render Field */
   glBindTexture(GL_TEXTURE_2D, texture);
   glEnable(GL_TEXTURE_2D);
   glPushMatrix();
      glBegin(GL_QUADS);
         glTexCoord2f(0, 0);
         glVertex3f(0, 0, 0);
         glTexCoord2f(0, texturePropY);
         glVertex3f(0, 0, sizeZ);
         glTexCoord2f(texturePropX, texturePropY);
         glVertex3f(sizeX, 0, sizeZ);
         glTexCoord2f(texturePropX, 0.0);
         glVertex3f(sizeX, 0,0);
      glEnd();
   glPopMatrix(); 
   glDisable(GL_TEXTURE_2D);

   /* Render Publicity */
   if(opt.getFieldDetail() > XUT_OPTIONS_FIELD_DETAIL_LOW)
   {
      publicities->render();
   }
}

/*********************************************************************
 *                          Static Variables                         *
 *********************************************************************/
fieldPub* field::publicities = NULL;
GLuint field::texture=0;
float field::texturePropX=1;
float field::texturePropY=1;
float field::sizeX=0;
float field::sizeZ=0;
float field::sideDelta=0;
float field::littleAreaDeltaX=0;
float field::littleAreaDeltaZ=0;
float field::penaltyMarkDelta=0;
