#ifndef _field_h
#define _field_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#define FIELD_Z    480        /**< Field Z size */
#define FIELD_X    736        /**< Field X size */
#define FIELD_SIDE_DELTA  32  /**< Field Inner Delta for all sides */
 
#define FIELD_LITTLE_AREA_DELTA_X   76 /**< Little Area X delta */
#define FIELD_LITTLE_AREA_DELTA_Z   59 /**< Little Area Z delta (from middle Z) */

#define FIELD_PENALTY_MARK_DELTA   150 /**< X Delta of penalty Mark */

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_opengl.h>
#include <string>
using namespace std;

#include "../etc/options.h"

#include "fieldpub.h"


/*! The field class */
class field
{
   public:
      /*! Constructor */
      field();
      /*! Destructor */
      ~field();

      /*! Create a field to use
       * \param fileName -> the fileName of the texture used for the field */
      void createField(string fileName);

      /*! Delete the created field */
      void deleteField();

      /*! Render the field to the screen */
      void render();

   protected:

      options opt;                   /**< The options */
   
      static fieldPub* publicities;  /**< The field publicities */

      static GLuint texture;         /**< The field texture used */
      static float texturePropX;     /**< Field Texture Proportion on X */
      static float texturePropY;     /**< Field Texture Proportion on Y */

      static float sizeX;            /**< Field X size */
      static float sizeZ;            /**< Field Z size */
      static float sideDelta;        /**< Field Inner Delta for all sides */
      static float littleAreaDeltaX; /**< Little Area X delta */
      static float littleAreaDeltaZ; /**< Little Area Z delta (from middle Z) */
      static float penaltyMarkDelta; /**< X Delta of penalty Mark */
};

#endif

