#ifndef _fieldpub_h
#define _fieldpub_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../etc/animodel.h"

/*! The publicity class is just a publicity on the field */
class publicity
{
   public:
      /*! Constructor
       * \param textureFile -> fileName of the texture used
       * \param x -> x position
       * \param y -> y position
       * \param z -> z position
       * \param alpha -> orientation angle */
      publicity(string textureFile, 
                float x, float y, float z, float alpha );
      /*! Destructor */
      ~publicity();

      /*! Get the Texture Used
       * \return the GL texture used */
      GLuint getTexture();

      /* Get publicity position
       * \param x -> x position
       * \param y -> y position
       * \param z -> z position
       * \param alpha -> orientation angle */
      void getPosition(float &x, float &y, float &z, float &alpha);

      publicity* next;        /**< Next Publicity on list */
      publicity* previous;    /**< Previous Publicity on list */

   protected:
      GLfloat posX;        /**< X position */
      GLfloat posY;        /**< Y position */
      GLfloat posZ;        /**< Z position */
      GLfloat angle;       /**< Orientation Angle */
      GLuint texture;      /**< GL texture used  */
};

/*! The publicity model is a list of publicities of various
 * textures using the same 3d model. */
class publicityModel
{
   public:
      /*! Constructor
       * \param fileName -> file name of the aniModel to load */
      publicityModel(string fileName);
      /*! Destructor */
      ~publicityModel();

      /* Add Publicity to the list 
       * \param textureName -> filename of the texture to use
       * \param x -> x position
       * \param y -> y position
       * \param z -> z position
       * \param alpha -> orientation angle */
      void addPublicity(string textureName,
                        float x, float y, float z, float alpha);

      /*! Render all Publicities */
      void render();

      /*! Get the aniModel fileName
       * \return -> animodel Filename */
      string getModelFile();

      publicityModel* next;      /**< Next on list */
      publicityModel* previous;  /**< Previous on list */

   protected:
      float modelMin[3],   /**< Disk Min Bounding box part */
            modelMax[3];   /**< Disk Max Bounding box part */

      string modelFile;    /**< The aniModel fileName */
      aniModel* model;     /**< aniModel used */

      publicity* list;     /**< List of publicities */
      int total;           /**< Total publicities on list */
      
};

/*! The fieldPub is publicities controller on the field */
class fieldPub
{
   public:
      /*! Constructor */
      fieldPub();
      /*! Destructor */
      ~fieldPub();

      /* Add Publicity to the list 
       * \param modelName -> filename of the aniModel used
       * \param textureName -> filename of the texture to use
       * \param x -> x position
       * \param y -> y position
       * \param z -> z position
       * \param alpha -> orientation angle */
      void addPublicity(string modelName, string textureName,
                        float x, float y, float z, float alpha);

      /*! Render */
      void render();

   protected:
      publicityModel* list;   /**< List of publicities models */
      int total;              /**< Total publicities models on list */
      
};


#endif

