
#include "main.h"

#include "../config.h"
#include "../etc/options.h"
#include "../etc/dirs.h"
#include "../etc/userinfo.h"
#include "../sound/sound.h"

/************************************************************
 *                     defineResolution                     *
 ************************************************************/
void defineResolution(SDL_Surface **screen, string title, bool worstCase=false) 
{   
   options opt;

   cout << "Initing Video..." << endl;

   SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
   SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8);

   if((!worstCase) && (opt.getAntiAliasing() > 0))
   {
      cout << "AntiAliasing: " << opt.getAntiAliasing() << endl; 
      SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
      SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, opt.getAntiAliasing());
   }
   else
   {
      cout << "AntiAliasing: Disabled." << endl;
      SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 0);
      SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 0);
   }
 
   /*SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
     SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
     SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5 );
     SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16 );
     SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1 );*/

   int flags = SDL_DOUBLEBUF | SDL_OPENGL;

   if(opt.getFullScreen())
   {
      cout << "Defined Fullscreen." << endl;
      flags |= SDL_FULLSCREEN;
   }

   *screen = SDL_SetVideoMode(opt.getWindowX(), opt.getWindowY(), 32, flags);
   if ( *screen == NULL ) 
   {
      cerr << "Oxi! Can't ajust video mode: " << SDL_GetError() << endl;
      if(!worstCase)
      {
         cerr << "Trying again at lower quality..." << endl;
         defineResolution(screen, title, true);
         return;
      }
      exit(2);
   }

   SDL_WM_SetCaption(title.c_str(),"");

}


/************************************************************
 *                        Farso_Init                        *
 ************************************************************/
void init(SDL_Surface **screen, string title)
{
   /* Start Openning the screen  */
   if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) 
   {
      printf("Argh! Can't init SDL!\n");
      exit(1);
   }
   atexit(SDL_Quit);

   /* Define the resolution */
   defineResolution(screen, title);

   /* Define ignored events */
   SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);
   SDL_EventState(SDL_ACTIVEEVENT, SDL_IGNORE);
   SDL_EventState(SDL_KEYDOWN, SDL_IGNORE);
   SDL_EventState(SDL_KEYUP, SDL_IGNORE);
   SDL_EventState(SDL_MOUSEMOTION, SDL_IGNORE);
   SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_IGNORE);
   SDL_EventState(SDL_MOUSEBUTTONUP, SDL_IGNORE);
   SDL_EventState(SDL_JOYAXISMOTION, SDL_IGNORE);
   SDL_EventState(SDL_JOYBALLMOTION, SDL_IGNORE);
   SDL_EventState(SDL_JOYHATMOTION, SDL_IGNORE);
   SDL_EventState(SDL_JOYBUTTONDOWN, SDL_IGNORE);
   SDL_EventState(SDL_JOYBUTTONUP, SDL_IGNORE);
   SDL_EventState(SDL_SYSWMEVENT, SDL_IGNORE);
   SDL_EventState(SDL_VIDEORESIZE, SDL_IGNORE);
   SDL_EventState(SDL_VIDEOEXPOSE, SDL_IGNORE);
   SDL_EventState(SDL_QUIT, SDL_IGNORE);
   SDL_EventState(SDL_USEREVENT, SDL_IGNORE);

   /* Init font controller */   
   dntFont fnt;
   cout << "Initing Fonts..." << endl;
   fnt.init();

   /* Init sound controller */
   sound snd;
   cout << "Initing Sound..." << endl;
   snd.init();

   cout << "Startup is done!" << endl;
}

/************************************************************
 *                         Farso_End                        *
 ************************************************************/
void end(SDL_Surface *screen)
{
   sound snd;
   dntFont fnt;

   /* finish font controller */
   cout << "Deleting fonts..." << endl;
   fnt.end();

   /* Finish sound controller */
   cout << "Finishing sounds..." << endl;
   snd.finish();

   cout << "Cleanup is done!" << endl;
}

/************************************************************
 *                           main                           *
 ************************************************************/
int main(int argc, char** argv)
{
   options opt;
   
   dirs dir;
   dir.findDataDirectories();

   userInfo info;
   info.getValuesFromSystem();

   xut* xutGame;
   SDL_Surface* screen;

   cout << "DNTeam XUT - " << VERSION << endl;

   /* Load Options */
   opt.load();
   opt.setLocale();

   /* Init the screen */
   init(&screen, "Xut");

   /* Create the engine */
   xutGame = new xut(screen, opt.getWindowX(), opt.getWindowY());

   xutGame->run();

   delete(xutGame);

   end(screen);

   return(1);
}

