/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rules.h"
#include "../gui/guiscore.h"
#include "../gui/guimessage.h"

#define BALL_ACTION_NONE       0
#define BALL_ACTION_SIDE       1
#define BALL_ACTION_BYLINE     2
#define BALL_ACTION_GOAL       3

/**********************************************************************
 *                             Constructor                            *
 **********************************************************************/
rules::rules()
{
}

/**********************************************************************
 *                              Destructor                            *
 **********************************************************************/
rules::~rules()
{
}

/**********************************************************************
 *                                clear                               *
 **********************************************************************/
void rules::clear()
{
   /* Time */
   halfMinutes = 0;
   halfSeconds = 0;
   timeStarted = SDL_GetTicks();

   /* Remaining Things */
   changedBallOwner = false;
   currentDisk = NULL;
   remainingGlobalTouches = maxRemainingGlobalTouches();
   remainingDiskTouches = 1; //Since middlefield kick

   /* Change the half */
   secondHalf = !secondHalf;
   state = RULE_STATE_MIDDLE;

}

/**********************************************************************
 *                      maxRemainingDiskTouches                     *
 **********************************************************************/
int rules::maxRemainingDiskTouches()
{
   /* To not change remaining at "stoppped" states */
   if(state == RULE_STATE_NORMAL)
   {
      //TODO implement other rules here in a switch!
      //switch(gameType)
      return(3);
   }
   return(remainingDiskTouches);
}

/**********************************************************************
 *                     maxRemainingGlobalTouches                    *
 **********************************************************************/
int rules::maxRemainingGlobalTouches()
{
   //TODO implement other rules here in a switch!
   return(12);
}

/**********************************************************************
 *                             changeTeamToAct                        *
 **********************************************************************/
void rules::changeTeamToAct()
{
   if(activeTeam == teamA)
   {
      activeTeam = teamB;
   }
   else
   {
      activeTeam = teamA;
   }
}

/**********************************************************************
 *                               getState                             *
 **********************************************************************/
int rules::getState()
{
   return(state);
}

/**********************************************************************
 *                            getActiveTeam                           *
 **********************************************************************/
team* rules::getActiveTeam()
{
   return(activeTeam);
}

/**********************************************************************
 *                            getUpperTeam                            *
 **********************************************************************/
team* rules::getUpperTeam()
{
   return(upperTeam);
}

/**********************************************************************
 *                               setBall                              *
 **********************************************************************/
void rules::setBall(ball* b)
{
   usedBall = b;
}

/**********************************************************************
 *                               setTeamA                             *
 **********************************************************************/
void rules::setTeamA(team* t)
{
   teamA = t;
}

/**********************************************************************
 *                               setTeamB                             *
 **********************************************************************/
void rules::setTeamB(team* t)
{
   teamB = t;
}

/**********************************************************************
 *                               setDisk                              *
 **********************************************************************/
bool rules::setDisk(teamPlayer* disk)
{
   if(currentDisk == disk)
   {
      if(remainingDiskTouches <= 0)
      {
         /* Can't Use this disk! */
         remainingDiskTouches = 0;
         return(false);
      }
   }
   else
   {
      /* Clear Disk Touches */
      currentDisk = disk;
      /* Reset disk touches */
      remainingDiskTouches = maxRemainingDiskTouches();
      if(remainingDiskTouches < 0)
      {
         remainingDiskTouches = 0;
      }
   }
   
   /* Decrease Disk Touches */
   remainingDiskTouches--;
   if(remainingDiskTouches < 0)
   {
      /* To avoid underflow  */
      remainingDiskTouches = 0;
   }
   /* Decrease Global Touches */
   remainingGlobalTouches--;

   return(true);
}

/**********************************************************************
 *                        getRemainingtouches                         *
 **********************************************************************/
int rules::getRemainingTouches()
{
   return(remainingGlobalTouches);
}

/**********************************************************************
 *                        getRemainingtouches                         *
 **********************************************************************/
int rules::getRemainingTouches(teamPlayer* disk)
{
   if(disk == currentDisk)
   {
      /* Is the current, so */
      return(remainingDiskTouches);
   }

   /* Isn't the current, so get max remaining */
   int res = maxRemainingDiskTouches();

   /* Make shure the remaining disk touches is lesser or equal to global */
   if(res > remainingGlobalTouches)
   {
      res = remainingGlobalTouches;
   }
   return(res);
}


/*********************************************************************
 *                          setPositions                             *
 *********************************************************************/
void rules::setPositions()
{
   bool isUpper = (upperTeam == teamA);
   bool isActing = (activeTeam == teamA);

   float midX = FIELD_X / 2.0;
   float midZ = FIELD_Z / 2.0;

   float x=0, z=0;

   switch(state)
   {
      case RULE_STATE_MIDDLE:
      {
         teamA->startPositionAtField(isUpper, isActing);
         teamB->startPositionAtField(!isUpper, !isActing);
         /* Put Ball At Center */
         usedBall->setPosition(midX, 0.0, midZ);
         usedBall->setVelocity(0.0, 0.0, 0.0);
      }
      break;

      case RULE_STATE_CORNER_KICK:
      {
         /* Define X Position */
         if(ballUpper)
         {
            x = FIELD_X - FIELD_SIDE_DELTA;
         }
         else
         {
            x = FIELD_SIDE_DELTA;
         }
         
         /* Define Z position */
         if(pZ >= midZ)
         {
            z = FIELD_Z - FIELD_SIDE_DELTA;
         }  
         else
         {
            z = FIELD_SIDE_DELTA;
         }

         /* Put Ball At Corner */
         usedBall->setPosition(x, 0.0, z);

         //TODO remove all players from the ball position!
      }
      break;

      case RULE_STATE_THROW_IN:
      {
         /* Put Ball At Exit Position */
         usedBall->setPosition(pX, 0.0, pZ);

         //TODO remove all players from the ball position!
      }
      break;

      case RULE_STATE_GOAL_KICK:
      {
         /* Define X Position */
         if(ballUpper)
         {
            x = FIELD_X - FIELD_LITTLE_AREA_DELTA_X;
         }
         else
         {
            x = FIELD_LITTLE_AREA_DELTA_X;
         }
         
         /* Define Z position */
         if(pZ >= midZ)
         {
            z = midZ + FIELD_LITTLE_AREA_DELTA_Z;
         }  
         else
         {
            z = midZ - FIELD_LITTLE_AREA_DELTA_Z;
         }

         /* Put Ball At Corner */
         usedBall->setPosition(x, 0.0, z);

         //TODO remove all players from the ball position!
      }
      break;

      case RULE_STATE_FREE_KICK:
      {
         /* Put Ball At Foul Position */
         usedBall->setPosition(pX, 0.0, pZ);

         /* TODO Remove players from the kick area */
      }
      break;

      case RULE_STATE_PENALTY_KICK:
      {
         /* Set X Position */  
         if(pX <= midX)
         {
            x = FIELD_PENALTY_MARK_DELTA;
         }
         else
         {
            x = FIELD_X - FIELD_PENALTY_MARK_DELTA;
         }
         
         /* Put Ball At Penalty Mark */
         usedBall->setPosition(midX, 0.0, midZ);

         /* TODO Remove all Players from the Great Area! */
      }
      break;
   }
}

/**********************************************************************
 *                               newTurn                              *
 **********************************************************************/
team* rules::newTurn()
{
   collidedBallFirst = false;
   collidedOwnDiskFirst = false;
   collidedEnemyDiskFirst = false;
   willShoot = false;
   ballAction = BALL_ACTION_NONE;
   pX = -1;
   pZ = -1;

   lastBallCollided = activeTeam;

   return(activeTeam);
}

/**********************************************************************
 *                              startHalf                             *
 **********************************************************************/
void rules::startHalf()
{
   clear();

   /* Set first player to act */
   if(secondHalf)
   {
      activeTeam = teamA;
      upperTeam = teamB;
   }
   else
   {
      activeTeam = teamB;
      upperTeam = teamA;
   }

   /* Put everyone at middle state positions */
   setPositions();
}

/**********************************************************************
 *                             updateClock                            *
 **********************************************************************/
bool rules::updateClock()
{
   Uint32 time = SDL_GetTicks() - timeStarted;
   
   halfSeconds = ((time / 1000) % 60);
   halfMinutes = time / 60000;

   return(halfMinutes >= minutesPerHalf);
}

/**********************************************************************
 *                             setGameType                            *
 **********************************************************************/
void rules::setGameType(int type)
{
   gameType = type;
}

/**********************************************************************
 *                          setMinutesPerHalf                         *
 **********************************************************************/
void rules::setMinutesPerHalf(int minutes)
{
   minutesPerHalf = minutes;
}

/**********************************************************************
 *                             isFirstHalf                            *
 **********************************************************************/
bool rules::isFirstHalf()
{
   return(!secondHalf);
}

/**********************************************************************
 *                           prepareToShoot                           *
 **********************************************************************/
void rules::prepareToShoot()
{
   willShoot = true;
}

/**********************************************************************
 *                          diskCollideDisk                           *
 **********************************************************************/
void rules::diskCollideDisk(team* diskPlayer1, team* diskPlayer2,
                            float x, float z)
{
   if( (!collidedBallFirst) && 
       (!collidedOwnDiskFirst) && 
       (!collidedEnemyDiskFirst) &&
       ( (activeTeam == diskPlayer1) || 
         (activeTeam == diskPlayer2)) )
   {
      pX = x;
      pZ = z;
      if(diskPlayer1 != diskPlayer2)
      {
         collidedEnemyDiskFirst = true;
      }
      else
      {
         collidedOwnDiskFirst = true;
      }
   }
}

/**********************************************************************
 *                          ballCollideDisk                           *
 **********************************************************************/
void rules::ballCollideDisk(team* player)
{
   /* Define last player contact the ball */
   lastBallCollided = player;

   /* Define if activeTeam disk player collidedd with ball first */
   if( (ballAction == BALL_ACTION_NONE) && 
       (player == activeTeam) && 
       (!collidedOwnDiskFirst) && (!collidedEnemyDiskFirst) )
   {
      collidedBallFirst = true;
   }
}

/**********************************************************************
 *                          ballExitAtSide                            *
 **********************************************************************/
void rules::ballExitAtSide(float x, float z)
{
   if( (collidedBallFirst) && (ballAction == BALL_ACTION_NONE) )
   {
      ballAction = BALL_ACTION_SIDE;
      pX = x;
      pZ = z;
   }
}

/**********************************************************************
 *                         ballExitAtByline                           *
 **********************************************************************/
void rules::ballExitAtByline(bool upper, float z)
{
   if( (collidedBallFirst) && (ballAction == BALL_ACTION_NONE) )
   {
      ballAction = BALL_ACTION_BYLINE;
      pX = -1;
      pZ = z;
      ballUpper = upper;
   }
}

/**********************************************************************
 *                           ballEnterGoal                            *
 **********************************************************************/
void rules::ballEnterGoal(bool upper)
{
   if( (collidedBallFirst) && (ballAction == BALL_ACTION_NONE) )
   {
      ballAction = BALL_ACTION_GOAL;
      ballUpper = upper;
   }
}

/**********************************************************************
 *                         verifyGoalValid                            *
 **********************************************************************/
void rules::verifyGoalValid()
{
   guiScore gameScore;
   guiMessage gameMessage;

   /* Verify of witch team the goal is from */
   bool teamAGoal = ( ( (ballUpper) && (upperTeam != teamA) ) ||
                      ( (!ballUpper) && (upperTeam == teamA) ) );
   
   if(teamAGoal)
   {
      /* If auto goal, always valid. Otherwise, need to verify if 
       * tells taht will try a goal shoot */
      if( (activeTeam == teamB) || (willShoot) )
      {
         gameScore.goalTeamA();
         state = RULE_STATE_MIDDLE;
      }
      else
      {
         gameMessage.set(gettext("Invalid Goal!"));
         state = RULE_STATE_GOAL_KICK;
      }
   }
   else
   {
      /* If auto goal, always valid. Otherwise, need to verify if 
       * tells taht will try a goal shoot */
      if( (activeTeam == teamA) || (willShoot) )
      {
         gameScore.goalTeamB();
         state = RULE_STATE_MIDDLE;
      }
      else
      {
         gameMessage.set(gettext("Invalid Goal!"));
         state = RULE_STATE_GOAL_KICK;
      }
   }

   /* Reset Touches if goes to middle kick */
   if(state == RULE_STATE_MIDDLE)
   {
      remainingGlobalTouches = maxRemainingGlobalTouches();
      remainingDiskTouches = 1;
   }
}

/**********************************************************************
 *                         changedTeamToAct                           *
 **********************************************************************/
bool rules::changedTeamToAct()
{
   return(changedBallOwner);
}

/**********************************************************************
 *                        ballAtFinalPosition                         *
 **********************************************************************/
void rules::ballAtFinalPosition()
{
   /* The System is stable, so verify things to the next turn */
   guiMessage gameMessage;

   /* If shooted, always change team to act */
   changedBallOwner = willShoot;

   /* Verify Fouls */
   if(collidedEnemyDiskFirst)
   {
      //FIXME verify where (PENALTY KICK!)
      state = RULE_STATE_FREE_KICK;
      remainingDiskTouches = 1;
      remainingGlobalTouches = maxRemainingGlobalTouches();
      changedBallOwner = true;
      gameMessage.set(gettext("Foul!"));
   }

   /* Verify balls miss */
   else if( (!collidedBallFirst) )
   {
      /* Do Not Collided with anything, or only 
       * with his own players, so ball's owner change */
      state = RULE_STATE_NORMAL;
      changedBallOwner = true;
   }

   /* Verify Throw-ins */
   else if(ballAction == BALL_ACTION_SIDE)
   {
      state = RULE_STATE_THROW_IN;
      if(activeTeam == lastBallCollided)
      {
         changedBallOwner = true;
      }
      remainingGlobalTouches = maxRemainingGlobalTouches();
      remainingDiskTouches = 1;
      gameMessage.set(gettext("Throw in!"));
   }

   /* Verify Corner Kicks and Goal Kicks */
   else if(ballAction == BALL_ACTION_BYLINE)
   {
      remainingGlobalTouches = maxRemainingGlobalTouches();
      remainingDiskTouches = 1;

      if(ballUpper) 
      {
         if(upperTeam == lastBallCollided)
         {
            state = RULE_STATE_CORNER_KICK;
         }
         else
         {
            state = RULE_STATE_GOAL_KICK;
         }
      }
      else
      {
         if(upperTeam != lastBallCollided)
         {
            state = RULE_STATE_CORNER_KICK;
         }
         else
         {
            state = RULE_STATE_GOAL_KICK;
         }
      }

      if(state == RULE_STATE_GOAL_KICK)
      {
         gameMessage.set(gettext("Goal Kick"));
      }
      else if(state == RULE_STATE_CORNER_KICK)
      {
         gameMessage.set(gettext("Corner Kick"));
      }
      
      changedBallOwner = (activeTeam == lastBallCollided);

   }

   /* Verify goals */
   else if(ballAction == BALL_ACTION_GOAL)
   {
      remainingGlobalTouches = maxRemainingGlobalTouches();
      remainingDiskTouches = 1;

      /* verify goal of who to change (or not) the owner */
      if(ballUpper) 
      {
         if(upperTeam != activeTeam)
         {
            changedBallOwner = true;
         }
      }
      else
      {
         if(upperTeam == activeTeam)
         {
            changedBallOwner = true;
         }
      }

      /* Verify if the goal was valid, changing the state
       *  to middle (valid) or goal kick (invalid) */
      verifyGoalValid();

   }

   /* Verify if global touches is underflowed */
   else if(remainingGlobalTouches <= 0)
   {
      state = RULE_STATE_NORMAL;
      gameMessage.set(gettext("No remaining moves!"));
      changedBallOwner = true;
   }

   /* Finally verify if collided with ball */
   else if(collidedBallFirst)
   {
      state = RULE_STATE_NORMAL;
   }

   /* Verify if the ball's owner changed */
   if(changedBallOwner)
   {
      remainingGlobalTouches = maxRemainingGlobalTouches();
      changeTeamToAct();
   }
}


/*  Static Variables   */

int rules::state;
int rules::minutesPerHalf = RULE_TIME_10;
int rules::gameType = RULE_TYPE_BALL_12;
int rules::halfMinutes;
int rules::halfSeconds;
Uint32 rules::timeStarted;
teamPlayer* rules::currentDisk;
bool rules::secondHalf = true;
bool rules::willShoot = false;
int rules::remainingGlobalTouches;
int rules::remainingDiskTouches;
team* rules::teamA = NULL;
team* rules::teamB = NULL;
team* rules::activeTeam = NULL;
team* rules::upperTeam = NULL;
team* rules::lastBallCollided = NULL;
ball* rules::usedBall = NULL;
bool rules::collidedBallFirst = false;
bool rules::collidedOwnDiskFirst = false;
bool rules::collidedEnemyDiskFirst = false;
int rules::ballAction = BALL_ACTION_NONE;
float rules::pX = -1;
float rules::pZ = -1;
bool rules::ballUpper = false;
bool rules::changedBallOwner = false;

