#ifndef _team_h
#define _team_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "teamplayer.h"
#include "field.h"

#include "../etc/animodel.h"

#include <SDL/SDL_opengl.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <string>
using namespace std;

/*! The team is the entity the user plays with. Each team is composed of 
 * 11 teamPlayers. 1 goal keeper and 10 disks. Each team has its
 * own models for the disks and for the goal keeper. */
class team
{
   public:

      /*! Constructor
       * \param fileName -> fileName of team to load */
      team(string fileName);
      /*! Destructor */
      ~team();

      /*! Put the team at a start position on the field.
       * \param upper -> true if the team is at upper side of the field, false
       *                 if at botton side.
       * \param withBall -> true if has ball pocession, false if has't. */
      void startPositionAtField(bool upper, bool withBall);

      /*! Render all the Team */
      void render();
   
      /*! Render all the team at pre-defined positions.
       * \param pos -> array of teamPlayer positions
       * \param angles -> array of teamPlayer angles */
      void render(float pos[11][2], float angles[11]);

      /*! Select a player at the position (+-5), if one exists.
       * \param x -> X coordinate of the player (+-5)
       * \param z -> Z coordinate of the player (+-5) 
       * \return -> pointer to the player or NULL, if no player found. */
      teamPlayer* selectPlayer(float x, float z);

      /*! Get the nearest player to the position
       * \param x -> x position 
       * \param z -> z position
       * \return -> pointer to the nearest player found */
      teamPlayer* getNearestPlayer(float x, float z);

      /*! Get the team's goal keeper
       * \return -> pointer to the goal keeper teamPlayer */
      teamPlayer* getGoalKeeper();

      /*! Get the team logo texture
       * \return OpenGL texture of the team Logo */
      GLuint getTeamLogo();

      /*! Get the team name
       * \return string with team name */
      string getTeamName();

      /*! The collision class is a friend, since it have to access 
       * all players to calculate dynamics. */
      friend class collision;
      
      /*! The collision class is a friend, since it have to access 
       * all players to get data for replay */
      friend class replay;
 
      float diskMin[3],          /**< Disk Min Bounding box part */
            diskMax[3];          /**< Disk Max Bounding box part */
      
      float goalKeeperMin[3],    /**< goalKeeper Min bounding box part */
            goalKeeperMax[3];    /**< goalKeeper Max bounding box part */

      aniModel* goalKeeperModel; /**< the goalKeeper 3D model */
            
   protected:
      /* Load a texture */
      void loadTexture(string strFilename, GLuint pId);
      void loadTexture(SDL_Surface* img, GLuint pId);

      /*! Create the texture for a disk
       * \param common -> pointer to the common texture
       * \param number -> player number
       * \param index -> disk index 
       * \param x -> x coordinate
       * \param y -> y coordinate */
      void createDiskTexture(SDL_Surface* common, int number, int index, 
                             int x, int y);

      string teamName;           /**< The Team Name */
      GLuint teamLogo;           /**< The Team Logo Texture */
      teamPlayer goalKeeper;     /**< The goal keeper */
      teamPlayer disk[10];       /**< The disks */

      GLuint goalKeeperTexture;  /**< Texture for the goal keeper */
      GLuint diskTexture[10];    /**< Each Disk Texture */

      aniModel* diskModel;       /**< The disk 3D model */


};


#endif

