#ifndef _cursor_h
#define _cursor_h

#include <SDL/SDL_opengl.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#include <string>
using namespace std;

#include "options.h"

/*************************************************************************
 *  DccNiTghtmare is public domain. Do whatever you want with this code. *
 *************************************************************************/

#define CURSOR_SELECTOR   0 /**< Selector Mouse Cursor */
#define CURSOR_USER_IMAGE 1 /**< User lodable Cursor */
#define CURSOR_TOTAL      2 /**< Total number of mouse cursors */

/*! A mouse cursor class.*/
class cursor
{
   public:
      /*! cursor Constructor */
      cursor();
      /*! cursor Destructor */
      ~cursor();  

      /*! Set current mouse Cursor 
       * \param nCursor -> cursor Number to use.*/
      void set(int nCursor);

      /*! Set the current mouse Cursor to an image
       * \param img -> sdl surface to set as cursor */
      void set(SDL_Surface* img);

      /*! Draw th Cursor to screen */
      void draw(int mouseX, int mouseY);

      /*! Gets the actual mouse cursor
       * \return -> pointer to the surface of the mouse cursor */
      int get();
      
   private:
      options opt;                  /**< Options info */
      GLuint texture[CURSOR_TOTAL]; /**< Cursors Textures */
      float sizeX[CURSOR_TOTAL];    /**< Cursors Widths */
      float sizeY[CURSOR_TOTAL];    /**< Cursors Heights */
      float propX[CURSOR_TOTAL];    /**< X Proportion */
      float propY[CURSOR_TOTAL];    /**< Y Proportion */
      int currentCursor;            /**< Current Cursor Index */
      
      /*! Load Cursor file 
       * \param fileName -> file name of cursor
       * \param index -> internal cursor index */
      void loadCursor(string fileName, int index);

      
};

#endif

