#ifndef _xut_opt_screen_h
#define _xut_opt_screen_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL_opengl.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#include "../etc/cursor.h"
#include "../etc/options.h"


/*! Text values table */
enum
{
   /* Main options screen texts */
   OPT_SCREEN_GRAPHICS=0,
   OPT_SCREEN_SOUND,
   OPT_SCREEN_GAME,
   OPT_SCREEN_KEYS,
   OPT_SCREEN_LANGUAGE,
   OPT_SCREEN_BACK,

   /* Graphics options texts */
   OPT_SCREEN_ANTI_ALIASING,
   OPT_SCREEN_RESOLUTION,
   OPT_SCREEN_FULLSCREEN,
   OPT_SCREEN_FIELD_DETAIL,
   OPT_SCREEN_CAMERA_TYPE,

   /* Sound Options texts */
   OPT_SCREEN_SOUND_VOLUME,

   /* Language Options textes */
   OPT_SCREEN_LANGUAGE_NAME,

   /* Key Options texts  */
   OPT_SCREEN_CAMERA_UP, 
   OPT_SCREEN_CAMERA_DOWN,
   OPT_SCREEN_CAMERA_LEFT,
   OPT_SCREEN_CAMERA_RIGHT,
   OPT_SCREEN_CAMERA_ROTATE_LEFT,
   OPT_SCREEN_CAMERA_ROTATE_RIGHT,
   OPT_SCREEN_CAMERA_ROTATE_UP,
   OPT_SCREEN_CAMERA_ROTATE_DOWN,
   OPT_SCREEN_CAMERA_ZOOM_UP,
   OPT_SCREEN_CAMERA_ZOOM_DOWN,
   OPT_SCREEN_WILL_SHOOT,
   OPT_SCREEN_REPLAY,
   OPT_SCREEN_ROTATE_LEFT,
   OPT_SCREEN_ROTATE_RIGHT,

   OPT_SCREEN_PRESS_KEY,

   OPT_SCREEN_TOTAL_TEXTURES

};

/*! Options Screen */
class optScreen
{
   public:
      /*! Constructor 
       * \param proj -> projection matrix
       * \param modl -> modelView Matrix
       * \param vPort -> viewPort vector 
       * \param bg -> background texture */
      optScreen(GLdouble* proj,GLdouble* modl, GLint* vPort, GLuint bg);

      /*! Destructor */
      ~optScreen();

      /*! Run the options screen
       * \param cursors -> current mouse cursors */
      void run(cursor* cursors);

   protected:
      /*! Draw the options Screen
       * \param cursors -> mouse cursors */
      void draw(cursor* cursors);

      /*! Render options
       * \param initial -> first options
       * \param end -> last option */
      void renderOptions(int initial, int end);

      /*! Load All Text Textures */
      void loadTextTextures();
      /*! reload All Text Textures */
      void reloadTextTextures();
      
      /*! load a single text texture  */
      void loadTextTexture(int index);

      /*! get the text value for the desired index */
      string getTextValue(int index);

      /*! verify input to the options screen
       * \return true when is to exit the options screen */
      bool verifyInput();
      
      void verifyMouseInput(int init, int end);
      void takeAction();
 
      options opt;            /**< The options */

      GLdouble* projection;   /**< Projection Matrix */
      GLdouble* modelView;    /**< Modelview Matrix */
      GLint* viewPort;        /**< view port vector */

      int state;              /**< Current State */

      int currentKeyCheck;    /**< Current key to get map */

      int mouseX,             /**< Current MouseX */
          mouseY;             /**< Current MouseY */
      Uint8 mButton;          /**< Current Mouse buttons state */

      int current;            /**< Current selected */
      GLuint bgTexture;       /**< Background texture */

      GLuint textTexture[OPT_SCREEN_TOTAL_TEXTURES]; /**< All Text textures */
};



#endif

