
/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "selector.h"
#include "dntfont.h"

#include "../lang/translate.h"

#include "../etc/draw.h"
#include "../etc/dirs.h"


#define SELECTOR_BUTTON_NONE      -1
#define SELECTOR_BUTTON_NEXT       0
#define SELECTOR_BUTTON_PREVIOUS   1
#define SELECTOR_BUTTON_ACCEPT     2
#define SELECTOR_BUTTON_CANCEL     3

#define SELECTOR_BUTTON_RADIUS    32


#define SELECTOR_STATE_NONE        0
#define SELECTOR_STATE_LOGO_OUT    1
#define SELECTOR_STATE_LOGO_IN     2
#define SELECTOR_STATE_SELECTED    3
#define SELECTOR_STATE_INITIAL     4

/****************************************************************
 *                          Constructor                         *
 ****************************************************************/
selector::selector(GLuint bg)
{
   state = SELECTOR_STATE_NONE;
   curTeam = 0;
   prevTeam = 0;
   currentButton = SELECTOR_BUTTON_NONE;
   selectedButton = SELECTOR_BUTTON_NONE;
   bgTexture = bg;

   FILE* file;
   dirs dir;
   int i;
   char buffer[512];
   string fileName;

   /* Load all teams infos for the selector */
   if(!(file = fopen(dir.getRealFile("teams/teams.lst").c_str(),"r")))
   {
      printf(gettext("Error while opening team file: %s\n"),"teams/teams.lst");
      return;
   }

   /* Get Quantity of Teams */
   if(fscanf(file, "%d", &totalTeams) == 0)
   {
      totalTeams = 0;
   }

   /* Alloc Teams Info */
   teams = new teamSel[totalTeams];

   SDL_Surface* img;

   for(i = 0; i < totalTeams; i++)
   {
      /* Get the fileName */
      if(fscanf(file, "%s ", buffer) == 1)
      {
         fileName = buffer;
         teams[i].fileName = fileName + ".xut";

         /* Load Images */
         glGenTextures(1, &teams[i].logo);
         img = IMG_Load(dir.getRealFile(fileName+"_symbol.png").c_str());
         if(img)
         {
            setTextureRGBA(img, teams[i].logo);
            SDL_FreeSurface(img);
         }
         else
         {
            printf(gettext("Couldn't Load Team Logo Image: %s\n"), 
                  (fileName + "_symbol.png").c_str());
         }

         /* Get the Team Name */
         if(fgets(buffer, sizeof(buffer), file) != NULL)
         {
            teams[i].name = translateDataString(buffer);
         }
      }
   }
   fclose(file);

   /* Generate OpenGL Textures for Buttons */
   glGenTextures(SELECTOR_TOTAL_BUTTONS, buttonTextures);

   /* Load All Buttons */
   loadButtonTexture(SELECTOR_BUTTON_NEXT, "gui/next_button.png");
   loadButtonTexture(SELECTOR_BUTTON_PREVIOUS, "gui/previous_button.png");
   loadButtonTexture(SELECTOR_BUTTON_ACCEPT, "gui/accept.png");
   loadButtonTexture(SELECTOR_BUTTON_CANCEL, "gui/cancel.png");
 
   /* Load Name Texture */
   glGenTextures(1,&nameTexture);
   nameImage = IMG_Load(dir.getRealFile("gui/sel_center.png").c_str());
   if(!nameImage)
   {
      printf(gettext("Couldn't load texture: %s\n"),"gui/sel_center.png");
   }
   reloadNameTexture();

   /* Generate the messageTexture */
   glGenTextures(1, &messageTexture);
   reloadMessageTexture();
}

/****************************************************************
 *                       loadButtonTexture                      *
 ****************************************************************/
void selector::loadButtonTexture(int button, string imageFile)
{
   dirs dir;
   SDL_Surface* img = IMG_Load(dir.getRealFile(imageFile).c_str());
   if(!img)
   {
      printf(gettext("Couldn't load texture: %s\n"), imageFile.c_str());
   }
   else
   {  
      setTextureRGBA(img, buttonTextures[button]);
      SDL_FreeSurface(img);
   }
}

/****************************************************************
 *                           Destructor                         *
 ****************************************************************/
selector::~selector()
{
   int i;

   /* Free Button Textures */ 
   glDeleteTextures(SELECTOR_TOTAL_BUTTONS, buttonTextures);

   /* Free Message Texture */
   glDeleteTextures(1, &messageTexture);

   /* Free Name Texture and Image */
   glDeleteTextures(1,&nameTexture);
   if(nameImage)
   {
      SDL_FreeSurface(nameImage);
   }
   for(i = 0; i < totalTeams; i++)
   {
      glDeleteTextures(1, &teams[i].logo);
   }
   delete []teams;
}

/****************************************************************
 *                            getTeamA                          *
 ****************************************************************/
string selector::getTeamA()
{
   return(teamA);
}

/****************************************************************
 *                            getTeamB                          *
 ****************************************************************/
string selector::getTeamB()
{
   return(teamB);
}

/****************************************************************
 *                      reloadNameTexture                       *
 ****************************************************************/
void selector::reloadNameTexture()
{
   if(!nameImage)
   {
      return;
   }

   dntFont fnt;
   fnt.defineFont(DNT_FONT_ARIAL, 14);
   fnt.defineFontAlign(DNT_FONT_ALIGN_CENTER);
   color_Set(255,0,0,255);
   SDL_Surface* tmp = SDL_ConvertSurface(nameImage, nameImage->format, 
                                         SDL_SWSURFACE);
   
   SDL_Rect rect;
   rect.x = 0;
   rect.y = 0;

   fnt.write(tmp, 0,20, teams[curTeam].name );

   setTextureRGBA(tmp, nameTexture);

   SDL_FreeSurface(tmp);
}

/****************************************************************
 *                    reloadMessageTexture                      *
 ****************************************************************/
void selector::reloadMessageTexture()
{
   if(!nameImage)
   {
      return;
   }

   dntFont fnt;
   fnt.defineFont(DNT_FONT_ARIAL, 14);
   fnt.defineFontAlign(DNT_FONT_ALIGN_CENTER);
   color_Set(255,122,150,255);
   SDL_Surface* tmp = SDL_ConvertSurface(nameImage, nameImage->format, 
                                         SDL_SWSURFACE);
   
   SDL_Rect rect;
   rect.x = 0;
   rect.y = 0;

   if(firstTeam)
   {
      fnt.write(tmp, 0,20, gettext("Select Team A"));
   }
   else
   {
      fnt.write(tmp, 0,20, gettext("Select Team B"));
   }
   setTextureRGBA(tmp, messageTexture);

   SDL_FreeSurface(tmp);
}

/****************************************************************
 *                          drawLogo                            *
 ****************************************************************/
void selector::drawLogo()
{
   float midY = opt.getWindowY() / 2.0;
   float midX = opt.getWindowX() / 2.0;

   /* The Initial Animation */
   if(state == SELECTOR_STATE_INITIAL)
   {
      glBindTexture(GL_TEXTURE_2D, teams[curTeam].logo);
      logoCurScale += 0.05;
      if(logoCurScale >= 1)
      {
         logoCurScale = 1;
         state = SELECTOR_STATE_NONE;
      }
   }

   /* Previous Logo OUT animation */
   else if(state == SELECTOR_STATE_LOGO_OUT)
   {
      glBindTexture(GL_TEXTURE_2D, teams[prevTeam].logo);
      if(curSignal)
      {
         logoCurAngle += 5;
      }
      else
      {
         logoCurAngle -= 5;
      }
      logoCurScale -= 0.1;
      if(logoCurScale <= 0)
      {
         logoCurScale = 0;
         state = SELECTOR_STATE_LOGO_IN;
      }
   }

   /* New Logo IN animation */
   else if(state == SELECTOR_STATE_LOGO_IN)
   {
      glBindTexture(GL_TEXTURE_2D, teams[curTeam].logo);
      if(curSignal)
      {
         logoCurAngle -= 5;
      }
      else
      {
         logoCurAngle += 5;
      }
      logoCurScale += 0.1;
      if(logoCurScale >= 1)
      {
         logoCurScale = 1;
         state = SELECTOR_STATE_NONE;
      }
   }

   /* Logo selected animation */
   else if(state == SELECTOR_STATE_SELECTED)
   {
      glBindTexture(GL_TEXTURE_2D, teams[prevTeam].logo);
      logoCurScale -= 0.05;

      if(curSignal)
      {
         logoCurAngle += 2;
         curSignal = logoCurAngle < 20;
      }
      else
      {
         logoCurAngle -= 2;
         curSignal = !(logoCurAngle > -20);
      }

      if(logoCurScale <= 0)
      {
         logoCurScale = 0;
         if(firstTeam)
         {
            /* Done with first team */
            firstTeam = false;
            /* So set the oponent */
            state = SELECTOR_STATE_INITIAL;
            reloadMessageTexture();
         }
         else
         {
            /* Done with selection */
            done = true;
            state = SELECTOR_STATE_NONE;
         }
      }
   }

   /* Normal State, without logo animations */
   else
   {
      glBindTexture(GL_TEXTURE_2D, teams[curTeam].logo);
      logoCurAngle = 0;
      logoCurScale = 1.0;
   }

   /* Finally, draw the logo */
   glPushMatrix();
      glTranslatef(midX, opt.getWindowY() - (midY-32), 0.0);
      glRotatef(logoCurAngle, 0, 0, 1);
      glScalef(logoCurScale, logoCurScale, 1.0);
      glBegin(GL_QUADS);
         glTexCoord2d(0,1);
         glVertex2f(-128,-128);
         glTexCoord2d(0,0);
         glVertex2f(-128,128);
         glTexCoord2d(1,0);
         glVertex2f(128,128);
         glTexCoord2d(1,1);
         glVertex2f(128,-128);
      glEnd();
   glPopMatrix();
}

/****************************************************************
 *                             draw                             *
 ****************************************************************/
void selector::draw(cursor* cursors)
{
   int i;
   float posX = 0, posY = 0;
   float midY = opt.getWindowY() / 2.0;
   float midX = opt.getWindowX() / 2.0;

   glClear (GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
   glClearColor(0.0, 0.0, 0.0, 1.0);

   draw2DMode();

   /* Draw Background */
   textureToScreen(bgTexture,0,0,opt.getWindowX(),opt.getWindowY(),800,600);

   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);
   glEnable(GL_TEXTURE_2D);

   /* Draw all buttons */
   for(i = 0; i < SELECTOR_TOTAL_BUTTONS; i++)
   {
      /* Get the center position */
      getButtonPosition(i, posX, posY);

      /* Draw the button */
      glBindTexture(GL_TEXTURE_2D, buttonTextures[i]);
      glPushMatrix();
         glTranslatef(posX, opt.getWindowY() - posY, 0.0);
         if(currentButton == i)
         {
            glScalef(0.8, 0.8, 1.0);
         }
         glBegin(GL_QUADS);
            glTexCoord2d(0,1);
            glVertex2f(-SELECTOR_BUTTON_RADIUS,-SELECTOR_BUTTON_RADIUS);
            glTexCoord2d(0,0);
            glVertex2f(-SELECTOR_BUTTON_RADIUS,SELECTOR_BUTTON_RADIUS);
            glTexCoord2d(1,0);
            glVertex2f(SELECTOR_BUTTON_RADIUS,SELECTOR_BUTTON_RADIUS);
            glTexCoord2d(1,1);
            glVertex2f(SELECTOR_BUTTON_RADIUS,-SELECTOR_BUTTON_RADIUS);
         glEnd();
      glPopMatrix();
   }

   /* Draw Team Name */
   glBindTexture(GL_TEXTURE_2D, nameTexture);
   glPushMatrix();
      glColor3f(1.0,1.0,1.0);
      glTranslatef(midX, opt.getWindowY() - (midY+128), 0.0);
      glBegin(GL_QUADS);
         glTexCoord2d(0,1);
         glVertex2f(-128,-32);
         glTexCoord2d(0,0);
         glVertex2f(-128,32);
         glTexCoord2d(1,0);
         glVertex2f(128,32);
         glTexCoord2d(1,1);
         glVertex2f(128,-32);
      glEnd();
   glPopMatrix();

   /* Draw which team is current selecting */
   glBindTexture(GL_TEXTURE_2D, messageTexture);
   glPushMatrix();
      glColor3f(1.0,1.0,1.0);
      glTranslatef(midX, opt.getWindowY() - (midY-188), 0.0);
      glBegin(GL_QUADS);
         glTexCoord2d(0,1);
         glVertex2f(-128,-32);
         glTexCoord2d(0,0);
         glVertex2f(-128,32);
         glTexCoord2d(1,0);
         glVertex2f(128,32);
         glTexCoord2d(1,1);
         glVertex2f(128,-32);
      glEnd();
   glPopMatrix();


   /* Draw Active Logo */
   drawLogo();

   /* Draw the Mouse Cursor */
   glPushMatrix();
      cursors->draw(mouseX, mouseY);
   glPopMatrix();

   glDisable(GL_TEXTURE_2D);
   glDisable(GL_BLEND);

   draw3DMode(FARVIEW);
   glFlush();
   SDL_GL_SwapBuffers();
}

/****************************************************************
 *                      getButtonPosition                       *
 ****************************************************************/
void selector::getButtonPosition(int button, float& posX, float& posY)
{
   /* Calculate Screen Middles */
   float midY = opt.getWindowY() / 2.0;
   float midX = opt.getWindowX() / 2.0;

   /* Define Positions */
   switch(button)
   {
      case SELECTOR_BUTTON_NEXT:
      {
         posX = midX+160;
         posY = midY+128;
      }
      break;
      case SELECTOR_BUTTON_PREVIOUS:
      {
         posX = midX-160;
         posY = midY+128;
      }
      break;
      case SELECTOR_BUTTON_ACCEPT:
      {
         posX = midX+192;
         posY = midY+192;
      }
      break;
      case SELECTOR_BUTTON_CANCEL:
      {
         posX = midX-192;
         posY = midY+192;
      }
      break;
   }
}

/****************************************************************
 *                           verifyIO                           *
 ****************************************************************/
void selector::verifyIO()
{
   int i;

   float posX=0, posY=0;

   Uint8 mButton;
   SDL_PumpEvents();
   mButton = SDL_GetMouseState(&mouseX, &mouseY);

   /* Verify Buttons */
   currentButton = SELECTOR_BUTTON_NONE;

   /* Buttons Only Avaible at IDLE state */
   if(state == SELECTOR_STATE_NONE)
   {
      for(i = 0; i < SELECTOR_TOTAL_BUTTONS; i++)
      {
         getButtonPosition(i, posX, posY);
         if(isPointIn(mouseX, mouseY, posX - SELECTOR_BUTTON_RADIUS,
                      posY - SELECTOR_BUTTON_RADIUS,
                      posX + SELECTOR_BUTTON_RADIUS,
                      posY + SELECTOR_BUTTON_RADIUS))
         {
            currentButton = i;
            if(mButton & SDL_BUTTON(1))
            {
               selectedButton = i;
            }
            else if(selectedButton == i)
            {
               /* Do action! */
               switch(selectedButton)
               {
                  case SELECTOR_BUTTON_NEXT:
                  {
                     prevTeam = curTeam;
                     curTeam = (curTeam+1) % totalTeams;
                     /* To avoid select teamB == teamA */
                     if( (!firstTeam) && (teams[curTeam].fileName == teamA))
                     {
                        curTeam = (curTeam+1) % totalTeams;
                     }
                     reloadNameTexture();
                     beginAnimation();
                     curSignal = true;
                  }
                  break;
                  case SELECTOR_BUTTON_PREVIOUS:
                  {
                     prevTeam = curTeam;
                     curTeam = (curTeam-1);
                     if(curTeam < 0)
                     {
                        curTeam = totalTeams-1;
                     }
                     /* To avoid select teamB == teamA */
                     if( (!firstTeam) && (teams[curTeam].fileName == teamA))
                     {
                        curTeam = (curTeam-1);
                        if(curTeam < 0)
                        {
                           curTeam = totalTeams-1;
                        }
                     }
                     reloadNameTexture();                     
                     beginAnimation();
                     curSignal = false;
                  }
                  break;
                  case SELECTOR_BUTTON_CANCEL:
                  {
                     /* Cancel the selection */
                     result = SELECTOR_CANCELED;
                     done = true;
                  }
                  break;
                  case SELECTOR_BUTTON_ACCEPT:
                  {
                     if(firstTeam)
                     {
                        /* Select the first team */
                        teamA = teams[curTeam].fileName;
                        state = SELECTOR_STATE_SELECTED;
                        /* To avoid select again the same team */
                        prevTeam = curTeam;
                        curTeam = (curTeam+1) % totalTeams;
                        reloadNameTexture();
                     }
                     else
                     {
                        /* Select the second team and exit */
                        teamB = teams[curTeam].fileName;
                        state = SELECTOR_STATE_SELECTED;
                        result = SELECTOR_DONE;
                        prevTeam = curTeam;
                     }
                  }
                  break;
               }
               /* Deselect the button */
               selectedButton = SELECTOR_BUTTON_NONE;
            }
            else
            {
               selectedButton = SELECTOR_BUTTON_NONE;
            }
         }
      }
   }
}

/****************************************************************
 *                       beginAnimation                         *
 ****************************************************************/
void selector::beginAnimation()
{
   state = SELECTOR_STATE_LOGO_OUT;
   logoCurAngle = 0;
   logoCurScale = 1.0;
}

/****************************************************************
 *                              run                             *
 ****************************************************************/
int selector::run(cursor* cursors)
{
   int time, varTime, sleepTime;
   int lastTime = 0;

   glDisable(GL_FOG);
   glDisable(GL_LIGHTING);

   result = -1;
   done = false;
   firstTeam = true;
   reloadMessageTexture();
   state = SELECTOR_STATE_INITIAL;
   logoCurAngle = 0;
   logoCurScale = 0;

   while(!done)
   {
      time = SDL_GetTicks();
      varTime = time - lastTime;

      if( ((varTime)) >= UPDATE_RATE)
      {
         lastTime = time;
         draw(cursors);

         verifyIO();
      }
      else
      {
         /* Sleep until time pass */
         sleepTime = (int) ((UPDATE_RATE-1) - varTime);
         if( (sleepTime > 0) && (sleepTime < UPDATE_RATE))
         {
            SDL_Delay(sleepTime);
         }
      }
   }
   
   glEnable(GL_LIGHTING);
   glEnable(GL_FOG);
   return(result);
}




