/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "videocontroller.h"

#include "../etc/dirs.h"

#define VIDEO_CONTROLLER_BUTTON_HALF_SIZE  16

/*********************************************************************
 *                            Constructor                            *
 *********************************************************************/
videoController::videoController()
{
   currentButton = -1;

   /* Generate all Textures */
   glGenTextures(VIDEO_CONTROLLER_TOTAL_BUTTONS, textures);

   /* Load All Textures */
   loadButtonTexture(VIDEO_CONTROLLER_STOP, "gui/replay/stop.png");
   loadButtonTexture(VIDEO_CONTROLLER_PLAY, "gui/replay/play.png");
   loadButtonTexture(VIDEO_CONTROLLER_PAUSE, "gui/replay/pause.png");
   loadButtonTexture(VIDEO_CONTROLLER_BACKWARD, "gui/replay/backward.png");
   loadButtonTexture(VIDEO_CONTROLLER_FORWARD, "gui/replay/forward.png");
   loadButtonTexture(VIDEO_CONTROLLER_HALF_SPEED, "gui/replay/12x.png");
   loadButtonTexture(VIDEO_CONTROLLER_NORMAL_SPEED, "gui/replay/1x.png");
   loadButtonTexture(VIDEO_CONTROLLER_DOUBLE_SPEED, "gui/replay/2x.png");
   loadButtonTexture(VIDEO_CONTROLLER_SHUTDOWN, "gui/replay/shutdown.png");
}


/*********************************************************************
 *                             Destructor                            *
 *********************************************************************/
videoController::~videoController()
{
   /* Delete all textures */
   glDeleteTextures(VIDEO_CONTROLLER_TOTAL_BUTTONS, textures);
}

/****************************************************************
 *                       loadButtonTexture                      *
 ****************************************************************/
void videoController::loadButtonTexture(int button, string imageFile)
{
   dirs dir;
   SDL_Surface* img = IMG_Load(dir.getRealFile(imageFile).c_str());
   if(!img)
   {
      printf(gettext("Couldn't load texture: %s\n"), imageFile.c_str());
   }
   else
   {  
      setTextureRGBA(img, textures[button]);
      SDL_FreeSurface(img);
   }
}

/****************************************************************
 *                      getButtonPosition                       *
 ****************************************************************/
void videoController::getButtonPosition(int button, float& posX, float& posY)
{
   int SCREEN_Y = opt.getWindowY();
   posX = 0;
   posY = 0;
   /* Define Positions */
   switch(button)
   {
      /* First line */
      case VIDEO_CONTROLLER_HALF_SPEED:
      {
         posX = 10 + VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
         posY = SCREEN_Y - 10 - VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
      }
      break;
      case VIDEO_CONTROLLER_NORMAL_SPEED:
      {
         posX = 20 + 3*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
         posY = SCREEN_Y - 10 - VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
      }
      break;
      case VIDEO_CONTROLLER_DOUBLE_SPEED:
      {
         posX = 30 + 5*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
         posY = SCREEN_Y - 10 - VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
      }
      break;
   
      /* Second Line */
      case VIDEO_CONTROLLER_PAUSE:
      {
         posX = 20 + 3*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
         posY = SCREEN_Y - 20 - 3*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
      }
      break;
   
      /* Third Line */
      case VIDEO_CONTROLLER_BACKWARD:
      {
         posX = 10 + VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
         posY = SCREEN_Y - 30 - 5*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
      }
      break;
      case VIDEO_CONTROLLER_PLAY:
      {
         posX = 20 + 3*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
         posY = SCREEN_Y - 30 - 5*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
      }
      break;
      case VIDEO_CONTROLLER_FORWARD:
      {
         posX = 30 + 5*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
         posY = SCREEN_Y - 30 - 5*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
      }
      break;

      /* Forth Line */
      case VIDEO_CONTROLLER_STOP:
      {
         posX = 20 + 3*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
         posY = SCREEN_Y - 40 - 7*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
      }
      break;
   
      /* Sixth Line */
      case VIDEO_CONTROLLER_SHUTDOWN:
      {
         posX = 10 + VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
         posY = SCREEN_Y - 50 - 9*VIDEO_CONTROLLER_BUTTON_HALF_SIZE;
      }
      break;
   }
}

/****************************************************************
 *                           render                             *
 ****************************************************************/
void videoController::render()
{
   int i;
   float posX=0, posY=0;

   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);
   glEnable(GL_TEXTURE_2D);

   for(i = 0; i < VIDEO_CONTROLLER_TOTAL_BUTTONS; i++)
   {
      getButtonPosition(i, posX, posY);

      /* Draw the button */
      glBindTexture(GL_TEXTURE_2D, textures[i]);
      glPushMatrix();
         glTranslatef(posX, opt.getWindowY() - posY, 0.0);
         if(currentButton == i)
         {
            glScalef(0.8, 0.8, 1.0);
         }
         glBegin(GL_QUADS);
            glTexCoord2d(0,1);
            glVertex2f(-VIDEO_CONTROLLER_BUTTON_HALF_SIZE,
                       -VIDEO_CONTROLLER_BUTTON_HALF_SIZE);
            glTexCoord2d(0,0);
            glVertex2f(-VIDEO_CONTROLLER_BUTTON_HALF_SIZE,
                       VIDEO_CONTROLLER_BUTTON_HALF_SIZE);
            glTexCoord2d(1,0);
            glVertex2f(VIDEO_CONTROLLER_BUTTON_HALF_SIZE,
                       VIDEO_CONTROLLER_BUTTON_HALF_SIZE);
            glTexCoord2d(1,1);
            glVertex2f(VIDEO_CONTROLLER_BUTTON_HALF_SIZE,
                       -VIDEO_CONTROLLER_BUTTON_HALF_SIZE);
         glEnd();
      glPopMatrix();
   }

   glDisable(GL_TEXTURE_2D);
   glDisable(GL_BLEND);
}

/****************************************************************
 *                        verifyButtons                         *
 ****************************************************************/
int videoController::verifyButtons(Uint8 mButton, int mouseX, int mouseY)
{
   int i;
   float posX=0, posY=0;
   bool done = false;
   currentButton = VIDEO_CONTROLLER_NOTHING;
   
   for(i = 0; ( (i < VIDEO_CONTROLLER_TOTAL_BUTTONS) && 
                (!done) ) ; i++)
   {
      getButtonPosition(i, posX, posY);

      /* Verify if mouse is inner button */
      if( (mouseX >= posX - VIDEO_CONTROLLER_BUTTON_HALF_SIZE) &&
          (mouseX <= posX + VIDEO_CONTROLLER_BUTTON_HALF_SIZE) &&
          (mouseY >= posY - VIDEO_CONTROLLER_BUTTON_HALF_SIZE) &&
          (mouseY <= posY + VIDEO_CONTROLLER_BUTTON_HALF_SIZE) )
      {
         done = true;
         /* Verify if mouse button is pressed */
         if(mButton & SDL_BUTTON(1))
         {
            currentButton = i;
         }
      }
   }
   return(currentButton);
}

